# MATLAB/OCTAVE script for TDOA
#  based on PANOSDRRADIO //www.panoradio-sdr.de/
#  'pins.csv' is a list of DAB transmitters (using to fill up tx1, tx2, .. and lagNmeter
#     lagNmeter variable contains the 'perfect' calculated lag (in meters)
#       or, lagNmeter variable contains the observed lag, here in number of samples converted to meters
#     lagNmeters =   lag_sample2meters(  observed_lag ,fs) 
#     lagNmeters =   tx_distance_diff1N 

clear
clc
pkg load io

pins = csv2cell('pins.csv',";");
 
[ rx_ref_long rx_ref_lat  ] =  { 3.2.0000 50.00000}{:}; 
%[ tx1 tx1_long tx1_lat ] = pins( cellfun(@(x) isequal(x, 'Oostende'), pins(:,4)),4:6){:};
%[ tx2 tx2_long tx2_lat ] = pins( cellfun(@(x) isequal(x, 'Brugge '), pins(:,4)),4:6){:};
%[ tx3 tx3_long tx3_lat ] = pins( cellfun(@(x) isequal(x, 'Aalter '), pins(:,4)),4:6){:};   
[ tx1 tx1_long tx1_lat ] = pins( cellfun(@(x) isequal(x, 'GentEu'), pins(:,4)),4:6){:};
[ tx2 tx2_long tx2_lat ] = pins( cellfun(@(x) isequal(x, 'AalstE'), pins(:,4)),4:6){:};
%[ tx3_long tx3_lat ] = pins( cellfun(@(x) isequal(x, 'Kortrijk'), pins(:,4)),4:6){:};   
%[ tx3 tx3_long tx3_lat ] = pins( cellfun(@(x) isequal(x, 'Eeklo i'), pins(:,4)),4:6){:};  
[ tx3 tx3_long tx3_lat ] = pins( cellfun(@(x) isequal(x, 'Oudenaar'), pins(:,4)),4:6){:};   
[ tx4 tx4_long tx4_lat ] = pins( cellfun(@(x) isequal(x, 'Herzele '), pins(:,4)),4:6){:};  
[ tx5 tx5_long tx5_lat ] = pins( cellfun(@(x) isequal(x, 'Kortrijk'), pins(:,4)),4:6){:};

fs = 2048000; # samplerate SDR
geo_ref_lat  = mean([tx1_lat, tx2_lat, rx_ref_lat]);
geo_ref_long = mean([tx1_long, tx2_long, rx_ref_long]);

function [ x,y ] = latlong2xy( lat, long, ref_lat, ref_long )
  % converts lat/long coordinates to cartesian x,y, output in km
  % ref_lat and ref_long is the geodetic reference point for plane approximation of the earth surface

  earth_circumf = 40074; % kilometer
  y = (lat - ref_lat)/360 * earth_circumf;
  x = (long - ref_long)/360 * cos(ref_lat*pi/180) * earth_circumf;
end

function [ lag ] = lag_sec( dist )
   lag = dist/2.9126e+08; %3e8 ;
end

function [ lag ] = lag_samples( dist, fs )
   lag = fs*dist/2.9126e+08; %3e8 ;
end
function [ dist ] = lag_sample2meters( lagsample, fs )
   dist = lagsample*2.9126e+08/fs; %3e8
end
function [ dist ] = lag_sec2meters( lagsec )
   dist = lagsec*2.9126e+08; %3e8
end
function [ dist ] = lag_hz2meters( laghz )
   dist =  2.9126e+08/laghz
end

function [ dist ] = dist_latlong( lat1, long1, lat2, long2, ref_lat, ref_long )
%DIST_LATLONG calculates distance between two specified points in meters
%accurate only in the (wide) area around the geodetic reference point ref_lat/_long

    [x1, y1] = latlong2xy( lat1, long1, ref_lat, ref_long );
    [x2, y2] = latlong2xy( lat2, long2, ref_lat, ref_long );

    dist = 1000*abs(  sqrt( (x1-x2)^2 + (y1-y2)^2 ) );
end


% known signal path differences between two txes to Ref (sign of result is important!)


disp(' ');
disp('______________________________________________________________________________________________');
disp([ 'Afstand ' tx1 '<->referentie<-> ' tx2] )
tx_distance_diff12a = dist_latlong(rx_ref_lat, rx_ref_long, tx1_lat, tx1_long, geo_ref_lat, geo_ref_long) ;
tx_distance_diff12b = dist_latlong(tx2_lat, tx2_long, rx_ref_lat, rx_ref_long, geo_ref_lat, geo_ref_long); 
tx_distance_diff12 = abs ( tx_distance_diff12a - tx_distance_diff12b );

disp( [ 'ref tot (tx1) ' num2str( tx_distance_diff12a ) ] )
disp( [ 'ref tot (tx2) ' num2str( tx_distance_diff12b ) ] )
disp([' (tx1 min tx2)' num2str(tx_distance_diff12)  'm  ' num2str(lag_sec(tx_distance_diff12)) 's peak: ' num2str(lag_samples(tx_distance_diff12,fs)) ' samples '  ] );
lag1meters =   lag_sample2meters(52,fs) % tx_distance_diff12 % 53 54 %

disp(' ');
disp('______________________________________________________________________________________________');
disp([ 'Afstand ' tx1 '<->referentie<-> ' tx3 ] )
tx_distance_diff13a = dist_latlong(rx_ref_lat, rx_ref_long, tx1_lat, tx1_long, geo_ref_lat, geo_ref_long) ;
tx_distance_diff13b = dist_latlong(tx3_lat, tx3_long, rx_ref_lat, rx_ref_long, geo_ref_lat, geo_ref_long); 
tx_distance_diff13 = abs ( tx_distance_diff13a - tx_distance_diff13b );

disp( [ 'ref tot (tx1) ' num2str( tx_distance_diff13a ) ] )
disp( [ 'ref tot (tx3) ' num2str( tx_distance_diff13b ) ] )
disp([' (tx1 min tx3)' num2str(tx_distance_diff13)  'm  ' num2str(lag_sec(tx_distance_diff13)) 's peak: ' num2str(lag_samples(tx_distance_diff13,fs)) ' samples '  ] );
lag2meters =  lag_sample2meters(27 , fs) %tx_distance_diff13 %22


disp(' ');
disp('______________________________________________________________________________________________');
disp([ 'Afstand ' tx1 '<->referentie<-> ' tx4 ] )
tx_distance_diff14a = dist_latlong(rx_ref_lat, rx_ref_long, tx1_lat, tx1_long, geo_ref_lat, geo_ref_long) ;
tx_distance_diff14b = dist_latlong(tx4_lat, tx4_long, rx_ref_lat, rx_ref_long, geo_ref_lat, geo_ref_long); 
tx_distance_diff14 = abs ( tx_distance_diff14a - tx_distance_diff14b );

disp( [ 'ref tot (tx1) ' num2str( tx_distance_diff14a ) ] )
disp( [ 'ref tot (tx4) ' num2str( tx_distance_diff14b ) ] )
disp([' (tx1 min tx4)' num2str(tx_distance_diff14)  'm  ' num2str(lag_sec(tx_distance_diff14)) 's peak: ' num2str(lag_samples(tx_distance_diff14,fs)) ' samples '  ] );
lag3meters =  lag_sample2meters(9 , fs) %tx_distance_diff14 %22  vanuit brugge = 150 samples (er is idd een piek)

disp(' ');
disp('______________________________________________________________________________________________');
disp([ 'Afstand ' tx1 '<->referentie<-> ' tx5 ] )
tx_distance_diff15a = dist_latlong(rx_ref_lat, rx_ref_long, tx1_lat, tx1_long, geo_ref_lat, geo_ref_long) ;
tx_distance_diff15b = dist_latlong(tx5_lat, tx5_long, rx_ref_lat, rx_ref_long, geo_ref_lat, geo_ref_long); 
tx_distance_diff15 = abs ( tx_distance_diff15a - tx_distance_diff15b );

disp( [ 'ref tot (tx1) ' num2str( tx_distance_diff15a ) ] )
disp( [ 'ref tot (tx5) ' num2str( tx_distance_diff15b ) ] )
disp([' (tx1 min tx5)' num2str(tx_distance_diff15)  'm  ' num2str(lag_sec(tx_distance_diff15)) 's peak: ' num2str(lag_samples(tx_distance_diff15,fs)) ' samples '  ] );
lag4meters =  lag_sample2meters(127, fs) %tx_distance_diff15 %22  vanuit herzele = 9 samples (er is idd een piek)



disp(' ');
disp('______________________________________________________________________________________________');
disp('HTML');
rx_lat_positions  = [tx1_lat   tx2_lat   tx3_lat  tx4_lat  tx5_lat]; %rx_ref_lat ];
rx_long_positions = [tx1_long  tx2_long  tx3_long tx4_long tx5_long ]; %rx_ref_long];


disp(' ');
disp('______________________________________________________________________________________________');
disp('HYPERBOLAS/CIRCLE LAGS');

[points_lat1, points_long1] = gen_hyperbola(lag1meters,  tx2_lat, tx2_long, tx1_lat, tx1_long, geo_ref_lat, geo_ref_long); 
[points_lat2, points_long2] = gen_hyperbola(lag2meters,  tx3_lat, tx3_long, tx1_lat, tx1_long, geo_ref_lat, geo_ref_long); 
[points_lat3, points_long3] = gen_hyperbola(lag3meters,  tx4_lat, tx4_long, tx1_lat, tx1_long, geo_ref_lat, geo_ref_long); 
[points_lat4, points_long4] = gen_hyperbola(lag4meters,  tx5_lat, tx5_long, tx1_lat, tx1_long, geo_ref_lat, geo_ref_long); 
%[points_lat1, points_long1] = gen_circle(lag1meters,  tx2_lat, tx2_long,   geo_ref_lat, geo_ref_long); 
%[points_lat2, points_long2] = gen_circle(lag2meters,  tx1_lat, tx1_long,   geo_ref_lat, geo_ref_long); 


hyperbola_lat_cell  = {points_lat1,  points_lat2, points_lat3 , points_lat4};
hyperbola_long_cell = {points_long1, points_long2, points_long3, points_long4};

create_html_file_osm( ['out/map_'  'osm.html'], rx_lat_positions, rx_long_positions, hyperbola_lat_cell, hyperbola_long_cell);
disp('______________________________________________________________________________________________'); 

system("start out/map_osm.html")
